// Copyright 2011 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.apache.tapestry5.internal.services;

import org.apache.tapestry5.services.ComponentClassResolver;
import org.apache.tapestry5.services.InvalidationEventHub;
import org.apache.tapestry5.services.InvalidationListener;

/**
 * Split-out implementation of {@link InvalidationEventHub} that separates it from {@link ComponentInstantiatorSource}.
 * This was necessary due to some refactoring that made {@link ComponentClassResolver} and ComponentInstantiatorSource
 * mutually dependent.
 *
 * @since 5.3
 */
public interface InternalComponentInvalidationEventHub extends InvalidationEventHub
{
    /**
     * Invoked when the {@link ComponentInstantiatorSource} notices that
     * a class file within any of the controlled packages has changed. Fires the
     * {@link InvalidationListener#objectWasInvalidated()} notification.
     */
    void classInControlledPackageHasChanged();
}
