// Copyright 2010, 2011 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry5.internal.services.meta;

import org.apache.tapestry5.model.MutableComponentModel;
import org.apache.tapestry5.plastic.PlasticClass;
import org.apache.tapestry5.services.meta.MetaDataExtractor;
import org.apache.tapestry5.services.meta.MetaWorker;
import org.apache.tapestry5.services.transform.ComponentClassTransformWorker2;
import org.apache.tapestry5.services.transform.TransformationSupport;

import java.lang.annotation.Annotation;
import java.util.Map;

@SuppressWarnings("unchecked")
public class MetaWorkerImpl implements MetaWorker, ComponentClassTransformWorker2
{
    private final Map<Class, MetaDataExtractor> configuration;

    public MetaWorkerImpl(Map<Class, MetaDataExtractor> configuration)
    {
        this.configuration = configuration;
    }

    public ComponentClassTransformWorker2 getWorker()
    {
        return this;
    }

    public void transform(PlasticClass plasticClass, TransformationSupport support, MutableComponentModel model)
    {
        for (Class c : configuration.keySet())
        {
            Annotation annotation = plasticClass.getAnnotation(c);

            if (annotation != null)
            {
                MetaDataExtractor extractor = configuration.get(c);

                extractor.extractMetaData(model, annotation);
            }
        }
    }

}
