// Copyright 2009-2014 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry5.ioc.def;

import org.apache.tapestry5.ioc.AdvisorDef;
import org.apache.tapestry5.ioc.annotations.IncompatibleChange;

import java.util.Set;

/**
 * Extended version of {@link org.apache.tapestry5.ioc.def.ModuleDef} introduced in Tapestry 5.1 to allow for service
 * advisors (an improvement on service decorators).
 *
 * @since 5.1.0.0
 */
public interface ModuleDef2 extends ModuleDef
{
    /**
     * Returns all the service advisor definitions built/provided by this module.
     */
    Set<AdvisorDef> getAdvisorDefs();

    /**
     * Methods marked with @Startup are converted into Runnable instances and assigned here.
     *
     * @since 5.4
     */
    @IncompatibleChange(release = "5.4", details = "StartupDef replaces the artificial ContributionDef created for @Startup methods.")
    Set<StartupDef> getStartups();
}
