// Copyright 2006, 2011 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry5.ioc.services;

/**
 * Event hub for notifications when the IOC {@link org.apache.tapestry5.ioc.Registry} shuts down.
 */
public interface RegistryShutdownHub
{
    /**
     * Adds a listener for eventual notification.
     */
    void addRegistryShutdownListener(RegistryShutdownListener listener);

    /**
     * Adds a listener for eventual notification when the registry shuts down.  Runtime exceptions thrown by the listener
     * will be logged and ignored.
     *
     * @since 5.3
     */
    void addRegistryShutdownListener(Runnable listener);

    /**
     * Adds a listener for eventual notification. RegistryWillShutdownListeners are notified before any standard listeners,
     * and before service proxies and other parts of the Registry are  disabled. Runtime exceptions thrown by the listener will
     * be logged and ignored.
     *
     * @since 5.3
     */
    void addRegistryWillShutdownListener(Runnable listener);
}
